//
//  rn_DrawFramebufferTonemape.fsh
//  Rayne
//
//  Copyright 2013 by Überpixel. All rights reserved.
//  Unauthorized use is punishable by torture, mutilation, and vivisection.
//

#version 150
precision highp float;

uniform sampler2D targetmap0;
uniform float healthfactor;

in vec2 vertTexcoord;
out vec4 fragColor0;

//Rift
uniform vec2 LensCenter;
uniform vec2 ScreenCenter;
uniform vec2 Scale;
uniform vec2 ScaleIn;
uniform vec4 HmdWarpParam;
uniform vec4 ChromAbParam;

void main()
{
	vec2  theta = (vertTexcoord - LensCenter) * ScaleIn; // Scales to [-1, 1]
	float rSq = theta.x * theta.x + theta.y * theta.y;
	vec2  theta1 = theta * (HmdWarpParam.x + HmdWarpParam.y * rSq + HmdWarpParam.z * rSq * rSq + HmdWarpParam.w * rSq * rSq * rSq);

	// Detect whether blue texture coordinates are out of range since these will scaled out the furthest.
	vec2 thetaBlue = theta1 * (ChromAbParam.z + ChromAbParam.w * rSq);
	vec2 tcBlue = LensCenter + Scale * thetaBlue;
	if (!all(equal(clamp(tcBlue, ScreenCenter-vec2(0.25,0.5), ScreenCenter+vec2(0.25,0.5)), tcBlue)))
	{
		fragColor0 = vec4(0.0);
		return;
	}

	// Now do blue texture lookup.
	float blue = texture(targetmap0, tcBlue).b;

	// Do green lookup (no scaling).
	vec2  tcGreen = LensCenter + Scale * theta1;
	vec4  center = texture(targetmap0, tcGreen);

	// Do red scale and lookup.
	vec2  thetaRed = theta1 * (ChromAbParam.x + ChromAbParam.y * rSq);
	vec2  tcRed = LensCenter + Scale * thetaRed;
	float red = texture(targetmap0, tcRed).r;
	
	vec3 color0 = vec3(red, center.g, blue);
	
//	vec3 color0 = texture(targetmap0, vertTexcoord).rgb;

	fragColor0.rgb = mix(color0, vec3(1.0, 0.0, 0.0), clamp(rSq*rSq*healthfactor, 0.0, 1.0));
	fragColor0.a = 1.0;
}
